diff --git a/src/backend/access/transam/xlog.c b/src/backend/access/transam/xlog.c
index 190d86b..22b5ff9 100644
--- a/src/backend/access/transam/xlog.c
+++ b/src/backend/access/transam/xlog.c
@@ -6499,6 +6499,21 @@ StartupXLOG(void)
 		}
 		memcpy(&checkPoint, XLogRecGetData(record), sizeof(CheckPoint));
 		wasShutdown = (record->xl_info == XLOG_CHECKPOINT_SHUTDOWN);
+
+		/*
+		 * Make sure that REDO location exists and reset ReadRecPtr,
+		 * EndRecPtr to the REDO location.
+		 * Later, xlogctl->replayEndRecPtr, xlogctl->lastReplayedEndRecPtr
+		 * are set to these values as initial value.
+		 */
+		if (InArchiveRecovery && XLByteLT(checkPoint.redo, checkPointLoc))
+		{
+			if (!(record = ReadRecord(&(checkPoint.redo), LOG, false)))
+				ereport(FATAL,
+					(errmsg("could not find redo location referenced by checkpoint record from control file")));
+			EndRecPtr = ReadRecPtr;
+			ReadRecPtr = record->xl_prev;
+		}
 	}
 
 	LastRec = RecPtr = checkPointLoc;
